#!/usr/bin/python
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# (c) 2007 Red Hat. Written by skvidal@fedoraproject.org
# preupgrade-cli

import sys
import os
import os.path

sys.path.insert(0,'/usr/share/yum-cli/')
from cli import *
from utils import YumUtilBase
from yum import Errors
import preupgrade
from urlgrabber.grabber import URLGrabError
from output import YumTextMeter
        
class PreUpgradeCli(preupgrade.PreUpgrade, YumUtilBase):
    NAME = 'PreUpgrade' 
    USAGE = '"usage: preupgrade [options] release [path]'

    def __init__(self):
        preupgrade.PreUpgrade.__init__(self)
        YumUtilBase.__init__(self,
                             PreUpgradeCli.NAME,
                             preupgrade.__version__,
                             PreUpgradeCli.USAGE)
        self.main()

    def _valid_entries(self):
        cur_ver = float(self.conf.yumvar['releasever'])    
        print "valid entries include:"
        for rel in self.return_available_versions(cur_ver, beta_ok=True):
            print '   "%s"' % rel

    def getconfig(self):
        # Add util commandline options to the yum-cli ones
        parser = self.getOptionParser() 
        # Parse the commandline option and setup the basics.
        opts = self.doUtilConfigSetup()
        if len(self.cmds) < 1:
            print "please give a release to try to pre-upgrade to"
            self._valid_entries()
            sys.exit(1)
        myrelease = self.cmds[0]
        if not self.release_map.has_key(myrelease):
            self.errorprint("no release named %s available" % myrelease)
            self._valid_entries()
            sys.exit(1)
        return myrelease
    
    def errorprint(self, msg):
        print >> sys.stderr, 'Error: %s' % msg
        
    def main(self):
        myrelease = self.getconfig()
        # Step 1: set up yum
        self.setup(myrelease)

        try:
            self.setup_update_repo()
        except Errors.YumBaseError, e:
            self.errorprint(str(e))
            sys.exit(1)
            
        # setup progress callbacks on the enabled repos
        if sys.stdout.isatty():         
            self.setupProgressCallbacks()
        
        try:
            self.figure_out_what_we_need()
        except Errors.YumBaseError,e:
            self.errorprint(str(e))
            sys.exit(1)
            
        downloadpkgs = self.get_download_pkgs()
        del(self.tsInfo)
        
        self.listPkgs(downloadpkgs, 'Packages we need to download', 'list')
        # print out the size to download
        self.reportDownloadSize(downloadpkgs)
        
        # Step 3: download the rpms we need
        # confirm with user
        print 'Download packages? '
        if self._promptWanted():
            if not self.userconfirm():
                print 'Exiting on user command'
                sys.exit(1)
        # FIXME - check the packages? Durrrrrrrrrrrr
        problems = self.downloadPkgs(downloadpkgs) 

        # Step 4: Get boot files and modify bootloader
        self.retrieve_treeinfo()

        # let the user ctrl-C :)
        self.instrepo.setInterruptCallback(self.interrupt_callback)
        # setup progress callback for kernel/stage2 "Repo"        
        if sys.stdout.isatty(): 
            self.instrepo.setCallback(YumTextMeter(fo=sys.stdout))
        
        try:
            self.retrieve_critical_boot_files()
        except URLGrabError, e:
            # FIXME: allow retry, give details (e.g. not enough space)
            # buttons: "Quit" "Try again" "Different Mirror" etc.
            self.errorprint(str(e))
            sys.exit(1)

        stage2success = False
        try:
            stage2file = self.retrieve_non_critical_files()
            stage2_abs = self.bootpath+"/"+stage2file
            stage2success = True
        except preupgrade.PUError, e:
            self.errorprint(str(e))
                                
        extra_args = ""
        if stage2success:            
            (mdev,mpath) = preupgrade.bootpath_to_anacondapath(stage2_abs)
            if mdev == None:
                # preupgrade.st_dev_to_devicename must have failed. Blarg!
                self.errorprint("Could not determine device name for %s" % self.bootpath)
                # FIXME: tell user where to file a bug and what info to include?
            else:
                extra_args = "stage2=hd:%s:%s" % (mdev,mpath)
            
        print "Preparing system to boot into installer"
        # method=URL is needed as a backup way of fetching stage2/metadata,
        # just in case stuff changed between when we started and when they
        # finally reboot
        extra_args += " method=%s" % self.instrepo.urls[0]
        self.make_it_boot(extra_args)

        print 'All finished. The upgrade will begin when you reboot.'
        # FIXME: Ask and reboot for them

                        
if __name__ == '__main__':
    if os.getuid() == 0:
        util = PreUpgradeCli()
    else:
        print >> sys.stderr, "You need to be root to run this."
        sys.exit(1)
